import sys
from pathlib import Path
import json
import pytest

sys.path.insert(0, str(Path(__file__).resolve().parent.parent))

import giorgio.project_manager as pm
from giorgio.project_manager import initialize_project, create_script, upgrade_project


@pytest.fixture
def temp_project(tmp_path):
    """
    Create a temporary directory for each test.

    :param tmp_path: The temporary path provided by pytest.
    :type tmp_path: Path
    :return: Path to the temporary project directory.
    :rtype: Path
    """
    
    project_root = tmp_path / "my_giorgio_project"
    project_root.mkdir()
    
    return project_root


def test_initialize_project_creates_structure(temp_project):
    # Run initialize_project
    initialize_project(temp_project, project_name="TestProject")

    # Verify 'scripts/' exists
    scripts_dir = temp_project / "scripts"
    assert scripts_dir.is_dir(), "scripts/ directory was not created."

    # Verify 'modules/' exists with __init__.py
    modules_dir = temp_project / "modules"
    assert modules_dir.is_dir(), "modules/ directory was not created."
    assert (modules_dir / "__init__.py").is_file(), "__init__.py in modules/ missing."

    # Verify '.env' exists and is empty
    env_file = temp_project / ".env"
    assert env_file.is_file(), ".env file was not created."
    assert env_file.stat().st_size == 0, ".env file should be empty upon init."

    # Verify '.giorgio/config.json' exists with correct keys
    giorgio_dir = temp_project / ".giorgio"
    assert giorgio_dir.is_dir(), ".giorgio directory was not created."

    config_file = giorgio_dir / "config.json"
    assert config_file.is_file(), "config.json was not created."

    data = json.loads(config_file.read_text(encoding="utf-8"))
    assert "giorgio_version" in data, "giorgio_version not in config.json"
    assert "module_paths" in data, "module_paths not in config.json"
    assert data.get("module_paths") == ["modules"]
    assert data.get("project_name") == "TestProject"


def test_initialize_project_errors_if_already_exists(temp_project):
    # First initialization succeeds
    initialize_project(temp_project)

    # Second call should raise FileExistsError
    with pytest.raises(FileExistsError):
        initialize_project(temp_project)


def test_create_script_happy_path(temp_project):
    # First, initialize the project
    initialize_project(temp_project)

    # Create a new script at "foo/bar"
    create_script(temp_project, "foo/bar")

    script_folder = temp_project / "scripts" / "foo" / "bar"
    assert script_folder.is_dir(), "Script folder was not created."

    # Check __init__.py at each level: scripts/foo/__init__.py and scripts/foo/bar/__init__.py
    assert (temp_project / "scripts" / "foo" / "__init__.py").is_file()
    assert (script_folder / "__init__.py").is_file()

    # Check script.py exists
    script_file = script_folder / "script.py"
    assert script_file.is_file(), "script.py was not created."

    # Check that the placeholder was replaced (CONFIG name uses path)
    content = script_file.read_text(encoding="utf-8")
    assert "__SCRIPT_PATH__" not in content
    assert '"foo/bar"' in content, "CONFIG name placeholder not replaced."


def test_create_script_errors_if_scripts_dir_missing(temp_project):
    # Do not initialize: scripts/ does not exist
    with pytest.raises(FileNotFoundError):
        create_script(temp_project, "newscript")


def test_create_script_errors_if_already_exists(temp_project):
    initialize_project(temp_project)
    create_script(temp_project, "foo")

    with pytest.raises(FileExistsError):
        create_script(temp_project, "foo")


def test_upgrade_project_force_updates_version(temp_project, monkeypatch):
    # Initialize and write an old version in config.json
    initialize_project(temp_project)
    config_file = temp_project / ".giorgio" / "config.json"
    data = json.loads(config_file.read_text(encoding="utf-8"))
    data["giorgio_version"] = "0.0.1"
    config_file.write_text(json.dumps(data), encoding="utf-8")

    # Monkeypatch giorgio.project_manager._get_version to simulate installed
    # version "9.9.9"
    monkeypatch.setattr(
        "giorgio.project_manager._get_version",
        lambda pkg: "9.9.9"
    )

    # Monkeypatch questionary.confirm to always return True
    # This simulates user confirming the upgrade
    monkeypatch.setattr(
        pm.questionary,
        "confirm",
        lambda msg: type("StubQ", (), {"ask": lambda self: True})()
    )

    # Force update without validation
    upgrade_project(temp_project, force=True)

    updated = json.loads(config_file.read_text(encoding="utf-8"))
    assert updated["giorgio_version"] == "9.9.9"


def test_upgrade_project_validation_failure(temp_project, monkeypatch):
    # Initialize project and create an invalid script (missing CONFIG keys)
    initialize_project(temp_project)

    # Create scripts/bad/script.py with no CONFIG
    bad_folder = temp_project / "scripts" / "bad"
    bad_folder.mkdir(parents=True)
    (bad_folder / "__init__.py").touch()
    (bad_folder / "script.py").write_text("def run(context): pass\n", encoding="utf-8")

    # Monkeypatch importlib.metadata.version to a higher version
    monkeypatch.setattr(
        "giorgio.project_manager._get_version",
        lambda pkg: "9.9.9"
    )

    # Monkeypatch questionary.confirm to always return True
    # This simulates user confirming the upgrade
    monkeypatch.setattr(
        pm.questionary,
        "confirm",
        lambda msg: type("StubQ", (), {"ask": lambda self: True})()
    )

    with pytest.raises(RuntimeError):
        upgrade_project(temp_project, force=False)


def test_upgrade_project_happy_path(temp_project, monkeypatch):
    # Initialize project and create a valid script
    initialize_project(temp_project)

    good_folder = temp_project / "scripts" / "good"
    good_folder.mkdir(parents=True)
    (good_folder / "__init__.py").touch()

    # script.py has valid CONFIG
    (good_folder / "script.py").write_text(
        "CONFIG = {'name': 'good', 'description': 'desc'}\n"
        "PARAMS = {}\n"
        "def run(context): pass\n",
        encoding="utf-8"
    )

    # Monkeypatch installed version to "2.3.4"
    monkeypatch.setattr(
        "giorgio.project_manager._get_version",
        lambda pkg: "2.3.4"
    )

    # Monkeypatch questionary.confirm to always return True
    # This simulates user confirming the upgrade
    monkeypatch.setattr(
        pm.questionary,
        "confirm",
        lambda msg: type("StubQ", (), {"ask": lambda self: True})()
    )

    upgrade_project(temp_project, force=False)

    config_file = temp_project / ".giorgio" / "config.json"
    updated = json.loads(config_file.read_text(encoding="utf-8"))
    assert updated["giorgio_version"] == "2.3.4"



import sys
import json
from pathlib import Path

from typer.testing import CliRunner
import questionary

sys.path.insert(0, str(Path(__file__).resolve().parent.parent))

from giorgio.cli import app
import pytest
from typer.testing import CliRunner
from giorgio.cli import _parse_params, _discover_ui_renderers

runner = CliRunner()


def test_init_default(tmp_path, monkeypatch):
    monkeypatch.chdir(tmp_path)
    result = runner.invoke(app, ["init"])
    assert result.exit_code == 0
    assert (tmp_path / "scripts").is_dir()
    assert (tmp_path / "modules").is_dir()
    assert (tmp_path / ".giorgio").is_dir()
    
    config_path = tmp_path / ".giorgio" / "config.json"
    assert config_path.exists()
    
    cfg = json.loads(config_path.read_text())
    assert "giorgio_version" in cfg
    assert "module_paths" in cfg


def test_init_named(tmp_path, monkeypatch):
    monkeypatch.chdir(tmp_path)
    project_dir = tmp_path / "myproj"
    result = runner.invoke(app, ["init", "--name", str(project_dir)])
    assert result.exit_code == 0
    assert project_dir.is_dir()
    assert (project_dir / "scripts").is_dir()
    assert (project_dir / "modules").is_dir()


def test_new_script(tmp_path, monkeypatch):
    monkeypatch.chdir(tmp_path)
    runner.invoke(app, ["init"])
    result = runner.invoke(app, ["new", "myscript"])
    assert result.exit_code == 0
    
    script_dir = tmp_path / "scripts" / "myscript"
    assert script_dir.is_dir()
    assert (script_dir / "script.py").exists()


def test_new_exists_error(tmp_path, monkeypatch):
    monkeypatch.chdir(tmp_path)
    runner.invoke(app, ["init"])
    runner.invoke(app, ["new", "dup"])
    result = runner.invoke(app, ["new", "dup"])
    
    assert result.exit_code != 0
    assert "Error creating script" in result.stdout


def test_cli_run_and_parameters(tmp_path, monkeypatch):
    monkeypatch.chdir(tmp_path)
    runner.invoke(app, ["init"])
    
    # Create a script that requires an int param 'x'
    script_dir = tmp_path / "scripts" / "hello"
    script_dir.mkdir(parents=True)
    
    (script_dir / "__init__.py").write_text("", encoding="utf-8")
    (script_dir / "script.py").write_text(
        "PARAMS = {'x': {'type': int, 'required': True}}\n"
        "def run(context): print(context.params['x'])\n",
        encoding="utf-8",
    )
    
    # Missing param should error
    result = runner.invoke(app, ["run", "hello"])
    assert result.exit_code == 1
    assert "Missing required parameter" in result.stdout
    
    # Correct param prints value
    result = runner.invoke(app, ["run", "hello", "--param", "x=42"])
    assert result.exit_code == 0
    assert "42" in result.stdout


def test_cli_start(tmp_path, monkeypatch):
    monkeypatch.chdir(tmp_path)
    runner.invoke(app, ["init"])
    
    # Create a script with no params
    script_dir = tmp_path / "scripts" / "s"
    script_dir.mkdir(parents=True)
    
    (script_dir / "__init__.py").write_text("", encoding="utf-8")
    (script_dir / "script.py").write_text(
        "PARAMS = {}\ndef run(context): print('ok')\n", encoding="utf-8"
    )
    
    # Stub selection to 's'
    monkeypatch.setattr(
        questionary,
        "select",
        lambda *args, **kwargs: type("Q", (), {"ask": lambda self: "s"})(),
    )
    result = runner.invoke(app, ["start"])
    assert result.exit_code == 0
    assert "ok" in result.stdout


def test_parse_params_valid():
    params = ["foo=1", "bar=hello", "baz=3.14"]
    result = _parse_params(params)
    assert result == {"foo": "1", "bar": "hello", "baz": "3.14"}


def test_parse_params_empty():
    result = _parse_params([])
    assert result == {}


def test_parse_params_invalid_format():
    with pytest.raises(Exception):
        _parse_params(["foo", "bar=2"])


def test_discover_ui_renderers_returns_dict(monkeypatch):
    class DummyEP:
        def __init__(self, name):
            self.name = name
        def load(self):
            return str

    def fake_entry_points(group=None):
        if group == "giorgio.ui_renderers":
            return [DummyEP("dummy")]
        return []

    monkeypatch.setattr("giorgio.cli.entry_points", fake_entry_points)
    renderers = _discover_ui_renderers()
    assert isinstance(renderers, dict)
    assert "dummy" in renderers
    assert renderers["dummy"] is str


def test_discover_ui_renderers_handles_load_error(monkeypatch, capsys):
    class DummyEP:
        def __init__(self, name):
            self.name = name
        def load(self):
            raise RuntimeError("fail")

    def fake_entry_points(group=None):
        if group == "giorgio.ui_renderers":
            return [DummyEP("bad")]
        return []

    monkeypatch.setattr("giorgio.cli.entry_points", fake_entry_points)
    renderers = _discover_ui_renderers()
    assert "bad" not in renderers
    captured = capsys.readouterr()
    assert "Warning: could not load UI plugin" in captured.out



import re
from pathlib import Path
import setuptools

# The directory containing this file
HERE = Path(__file__).parent

# Read the long description from README.md if it exists (optional)
README = (HERE / "README.md").read_text(encoding="utf-8") if (HERE / "README.md").exists() else ""

# Extract the version from the __init__.py file
pkg_dir = Path(__file__).parent / "giorgio"
init_py = (pkg_dir / "__init__.py").read_text()
VERSION = re.search(r'__version__ = ["\']([^"\']+)["\']', init_py).group(1)

setuptools.setup(
    name="giorgio",
    version=VERSION,
    description="Giorgio: a micro-framework for interactive Python scripts",
    long_description=README,
    long_description_content_type="text/markdown",
    author="Danilo Musci",
    author_email="officina@musci.ch",
    url='https://github.com/officinaMusci/giorgio',
    packages=setuptools.find_packages(exclude=("tests",)),
    include_package_data=True,
    python_requires=">=3.8",
    install_requires=[
        "typer>=0.9.0",
        "questionary>=1.10.0",
        "python-dotenv>=1.0.0"
    ],
    entry_points={
        "console_scripts": [
            "giorgio=giorgio.cli:app",
        ],
        "giorgio.ui_renderers": [
            "cli = giorgio.ui_cli:CLIUIRenderer"
        ]
    },
    keywords='automation, cli, gui, scripts, micro-framework',
    project_urls={
       'Documentation': 'https://github.com/officinaMusci/giorgio#readme',
       'Bug Tracker': 'https://github.com/officinaMusci/giorgio/issues'
    },
    classifiers=[
        "Programming Language :: Python :: 3.8",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: MIT License"
    ]
)



import json
import sys
from pathlib import Path
from importlib.metadata import version as _get_version, PackageNotFoundError
import questionary
import importlib.util


def initialize_project(root: Path, project_name: str = None) -> None:
    """
    Initialize a Giorgio project at the given root path.

    Creates the following structure under root:
      - scripts/          (directory for user scripts)
      - modules/          (directory for shared modules, with __init__.py)
      - .env              (blank environment file)
      - .giorgio/         (configuration directory)
          - config.json   (project configuration)

    :param root: The root directory where the project will be initialized.
    :type root: Path
    :param project_name: Optional name for the project to be stored in
    config.json.
    :type project_name: str, optional
    :raises FileExistsError: If any of the required directories or files already
    exist.    
    """

    # Check/create the root directory
    root.mkdir(parents=True, exist_ok=True)

    # Create scripts/ (for user scripts)
    scripts_dir = root / "scripts"
    if scripts_dir.exists():
        raise FileExistsError(f"Directory '{scripts_dir}' already exists.")
    
    scripts_dir.mkdir()

    # Create modules/ (for shared modules) + __init__.py
    modules_dir = root / "modules"
    if modules_dir.exists():
        raise FileExistsError(f"Directory '{modules_dir}' already exists.")
    
    modules_dir.mkdir()
    
    # Create __init__.py inside to make 'modules' importable
    init_file = modules_dir / "__init__.py"
    init_file.touch()

    # Create .env (empty file)
    env_file = root / ".env"
    if env_file.exists():
        raise FileExistsError(f"File '{env_file}' already exists.")
    
    env_file.touch()

    # Create .giorgio/ and config.json
    giorgio_dir = root / ".giorgio"
    if giorgio_dir.exists():
        raise FileExistsError(f"Directory '{giorgio_dir}' already exists.")
    
    giorgio_dir.mkdir()

    config_file = giorgio_dir / "config.json"
    
    try:
        current_version = _get_version("giorgio")
    
    except PackageNotFoundError:
        current_version = "0.0.0"

    default_config = {
        "giorgio_version": current_version,
        "module_paths": ["modules"]
    }
    
    if project_name:
        default_config["project_name"] = project_name

    with config_file.open("w", encoding="utf-8") as f:
        json.dump(default_config, f, indent=2)


def create_script(root: Path, script_relative_path: str) -> None:
    """
    Scaffold a new script under the 'scripts/' directory.

    Creates a folder at scripts/<script_relative_path>/ containing:
      - __init__.py         (to allow importing if needed)
      - script.py           (with boilerplate read from a template)

    :param root: Path to the project root.
    :type root: Path
    :param script_relative_path: Relative path under 'scripts/' where the new
    script should be created (e.g., "data/clean").
    :type script_relative_path: str
    :raises FileNotFoundError: If the 'scripts/' directory does not exist.
    :raises FileExistsError: If the target script directory already exists.
    :raises FileNotFoundError: If the script template file does not exist.
    """

    scripts_dir = root / "scripts"
    if not scripts_dir.exists():
        raise FileNotFoundError(f"'scripts/' directory not found in {root}.")

    target_dir = scripts_dir / script_relative_path
    if target_dir.exists():
        raise FileExistsError(f"Script directory '{target_dir}' already exists.")

    # Create nested directories (data/clean, etc.)
    target_dir.mkdir(parents=True, exist_ok=False)
    
    # Create an __init__.py at each level to allow package import if needed
    # Example: scripts/data/__init__.py and scripts/data/clean/__init__.py
    parts = target_dir.relative_to(scripts_dir).parts
    cumulative = scripts_dir
    
    for part in parts:
        cumulative = cumulative / part
        init_path = cumulative / "__init__.py"
        
        if not init_path.exists():
            init_path.touch()

    # Create script.py from the template
    script_file = target_dir / "script.py"
    template_dir = Path(__file__).parent / "templates"
    template_file = template_dir / "script_template.py"
    if not template_file.exists():
        raise FileNotFoundError(f"Template file '{template_file}' not found.")

    # Read the template content and replace the __SCRIPT_PATH__ placeholder
    template_content = template_file.read_text(encoding="utf-8")
    boilerplate = template_content.replace("__SCRIPT_PATH__", script_relative_path)

    with script_file.open("w", encoding="utf-8") as f:
        f.write(boilerplate)


def upgrade_project(root: Path, force: bool = False) -> None:
    """
    Perform a project upgrade to the latest Giorgio version.

    - Reads the .giorgio/config.json file to get the current project version.
    - Compares it to the installed version of Giorgio.
    - If force=True: directly writes the new version to config.json.
    - Otherwise: performs a validation (dry-run) of all scripts under 'scripts/'.
      Each script is imported and it is checked that CONFIG contains 'name' and 'description'.
      If validation succeeds, the user is prompted to confirm the update, then the file is modified.
      
    :param root: Path to the project root.
    :type root: Path
    :param force: If True, skips validation and directly updates the version.
    :type force: bool
    :raises FileNotFoundError: If the configuration file or scripts directory does not exist.
    :raises PackageNotFoundError: If Giorgio is not installed.
    """
    
    giorgio_dir = root / ".giorgio"
    config_file = giorgio_dir / "config.json"
    scripts_dir = root / "scripts"

    if not config_file.exists():
        raise FileNotFoundError(f"Configuration file '{config_file}' not found.")
    
    if not scripts_dir.exists():
        raise FileNotFoundError(f"Scripts directory '{scripts_dir}' not found.")

    # Load the project version from config.json
    with config_file.open("r", encoding="utf-8") as f:
        config_data = json.load(f)
    
    project_version = config_data.get("giorgio_version", "0.0.0")

    # Get the installed version
    try:
        installed_version = _get_version("giorgio")
    
    except PackageNotFoundError:
        installed_version = "0.0.0"

    if project_version == installed_version and not force:
        print("Project is already up-to-date.")
        return

    def validate_scripts() -> bool:
        """
        Validate all scripts in the 'scripts/' directory by importing them
        and checking that their CONFIG contains 'name' and 'description'.

        :return: True if all scripts pass validation, False otherwise.
        :rtype: bool
        """
        failed = []
        
        for script_path in scripts_dir.rglob("script.py"):
            rel_path = script_path.relative_to(scripts_dir).parent
            spec_path = script_path

            try:
                # Temporarily add scripts_dir to sys.path
                sys.path.insert(0, str(scripts_dir))
                
                module_name = ".".join(rel_path.parts + ("script",))
                spec = importlib.util.spec_from_file_location(module_name, spec_path)
                module = importlib.util.module_from_spec(spec)
                spec.loader.exec_module(module)  # type: ignore

                # Check CONFIG
                cfg = getattr(module, "CONFIG", None)
                if not isinstance(cfg, dict) or "name" not in cfg or "description" not in cfg:
                    failed.append(str(rel_path))
            
            except Exception as e:
                failed.append(f"{rel_path} (error: {e})")
            
            finally:
                # Remove scripts_dir from sys.path if added
                if sys.path and sys.path[0] == str(scripts_dir):
                    sys.path.pop(0)

        if failed:
            print("Validation failed for the following scripts:")
            
            for fpath in failed:
                print(f"  - {fpath}")
            
            return False
        
        return True

    if force:
        confirm = True
    
    else:
        print(f"Current project version: {project_version}")
        print(f"Installed Giorgio version: {installed_version}")
        print("Running validation on all scripts...")
        
        if not validate_scripts():
            raise RuntimeError("Upgrade aborted due to validation failures.")
        
        # User confirmation
        confirm = questionary.confirm("All scripts validated successfully. Update project version?").ask()

    if confirm:
        config_data["giorgio_version"] = installed_version
        
        with config_file.open("w", encoding="utf-8") as f:
            json.dump(config_data, f, indent=2)
        
        print(f"Project upgraded to Giorgio version {installed_version}.")
    
    else:
        print("Upgrade canceled.")




import sys
from pathlib import Path
import os
import time
import signal
import threading
import pytest

sys.path.insert(0, str(Path(__file__).resolve().parent.parent))

from giorgio.execution_engine import ExecutionEngine, GiorgioCancellationError


def write_script(tmp_path: Path, name: str, content: str) -> None:
    """
    Write a script to the temporary path with the given name and content.

    :param tmp_path: The temporary path where the script will be written.
    :type tmp_path: Path
    :param name: The name of the script directory.
    :type name: str
    :param content: The content of the script file.
    :type content: str
    :return: None
    :rtype: None
    """
    
    script_dir = tmp_path / "scripts" / name
    script_dir.mkdir(parents=True, exist_ok=True)
    
    (script_dir / "__init__.py").write_text("", encoding="utf-8")
    (script_dir / "script.py").write_text(content, encoding="utf-8")


def test_run_no_params(tmp_path, capsys):
    write_script(tmp_path, "noparams", """
PARAMS = {}
def run(context):
    print("hello")
""")
    
    engine = ExecutionEngine(tmp_path)
    
    with pytest.raises(RuntimeError):
        engine.run_script("noparams", cli_args=None)
    
    engine.run_script("noparams", cli_args={})
    captured = capsys.readouterr()
    assert "hello" in captured.out


def test_required_param_missing(tmp_path):
    write_script(tmp_path, "required", """
PARAMS = {
    "x": {"type": int, "required": True}
}
def run(context):
    pass
""")
    
    engine = ExecutionEngine(tmp_path)
    with pytest.raises(RuntimeError):
        engine.run_script("required", cli_args={})


def test_required_param_provided(tmp_path, capsys):
    write_script(tmp_path, "provided", """
PARAMS = {
    "x": {"type": int, "required": True}
}
def run(context):
    print(context.params["x"])
""")
    
    engine = ExecutionEngine(tmp_path)
    engine.run_script("provided", cli_args={"x": "5"})
    captured = capsys.readouterr()
    assert captured.out.strip() == "5"


def test_default_param(tmp_path, capsys):
    write_script(tmp_path, "default", """
PARAMS = {
    "x": {"type": int, "default": 7}
}
def run(context):
    print(context.params["x"])
""")
    
    engine = ExecutionEngine(tmp_path)
    engine.run_script("default", cli_args={})
    captured = capsys.readouterr()
    assert captured.out.strip() == "7"


def test_invalid_type(tmp_path):
    write_script(tmp_path, "badtype", """
PARAMS = {
    "x": {"type": int, "required": True}
}
def run(context):
    pass
""")
    
    engine = ExecutionEngine(tmp_path)

    with pytest.raises(ValueError):
        engine.run_script("badtype", cli_args={"x": "abc"})


def test_invalid_choice(tmp_path):
    write_script(tmp_path, "choice", """
PARAMS = {
    "x": {"type": str, "choices": ["a", "b"], "required": True}
}
def run(context):
    pass
""")
    
    engine = ExecutionEngine(tmp_path)

    with pytest.raises(ValueError):
        engine.run_script("choice", cli_args={"x": "c"})


def test_env_default(tmp_path, capsys, monkeypatch):
    write_script(tmp_path, "env", """
PARAMS = {
    "x": {"type": str, "default": "${MYVAR}"}
}
def run(context):
    print(context.params["x"])
""")
    
    # simulate .env load
    monkeypatch.setenv("MYVAR", "hello_env")
    engine = ExecutionEngine(tmp_path)
    engine.env["MYVAR"] = "hello_env"
    engine.run_script("env", cli_args={})
    captured = capsys.readouterr()
    assert captured.out.strip() == "hello_env"


def test_add_params_forbidden(tmp_path):
    write_script(tmp_path, "add", """
PARAMS = {}
def run(context):
    context.add_params({"y": {"type": int}})
""")
    
    engine = ExecutionEngine(tmp_path)

    with pytest.raises(RuntimeError):
        engine.run_script("add", cli_args={})


def test_boolean_conversion(tmp_path, capsys):
    write_script(tmp_path, "bool", """
PARAMS = {
    "flag": {"type": bool, "required": True}
}
def run(context):
    print(context.params["flag"])
""")
    
    engine = ExecutionEngine(tmp_path)

    # false values
    for raw in ("false", "0", "no", "n"):
        engine.run_script("bool", cli_args={"flag": raw})
        captured = capsys.readouterr()
        assert captured.out.strip() == "False"

    # true values
    for raw in ("true", "1", "yes", "y"):
        engine.run_script("bool", cli_args={"flag": raw})
        captured = capsys.readouterr()
        assert captured.out.strip() == "True"


def test_noninteractive_requires_cli_args(tmp_path):
    write_script(tmp_path, "mustcli", """
PARAMS = {}
def run(context):
    pass
""")
    engine = ExecutionEngine(tmp_path)

    with pytest.raises(RuntimeError):
        engine.run_script("mustcli", cli_args=None)


def test_cancellation(tmp_path, capsys):
    write_script(tmp_path, "loop", """
import time
from giorgio.execution_engine import GiorgioCancellationError
PARAMS = {}
def run(context):
    try:
        while True:
            time.sleep(0.1)
            print("Running...")
    except GiorgioCancellationError:
        print("Script execution cancelled.")
""")
    pid = os.getpid()
    def send_sigint_after_delay():
        time.sleep(0.5)
        os.kill(pid, signal.SIGINT)
    threading.Thread(target=send_sigint_after_delay, daemon=True).start()

    engine = ExecutionEngine(tmp_path)
    engine.run_script("loop", cli_args={})
    out = capsys.readouterr().out
    assert "Script execution cancelled." in out


def test_signal_handler_raises(tmp_path):
    engine = ExecutionEngine(tmp_path)
    with pytest.raises(GiorgioCancellationError):
        engine._signal_handler(signal.SIGINT, None)


from giorgio.execution_engine import Context, GiorgioCancellationError

CONFIG = {
    "name": "__SCRIPT_PATH__",
    "description": ""
}

PARAMS = { }


def run(context: Context):
    try:
        # Your script logic goes here
        print("Running the script...")
    
    except GiorgioCancellationError:
        print("Execution was cancelled by the user.")
